/*
 * Copyright 2008 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 */


#ifndef DIMPRO_TIME_HPP_
#define DIMPRO_TIME_HPP_

#include "scalar.hpp"

namespace NSDimensionPromise
{

/**
 * Timeの次元定義.
 */
template <typename T>
struct TimeBaseTraits
{
    typedef ScalarNumber<T, 0, 0, 1, 0, 0, 0, 0> ScalarType;
};

/**
 * Timeを表すベースクラス.
 */
template <typename T>
class TimeBase : public TimeBaseTraits<T>::ScalarType
{
public:
    /**
     * Timeを表すスカラー変数を生成する.
     * @param sec 変数の初期値
     */
    explicit TimeBase(const T& sec = T())
        : TimeBaseTraits<T>::ScalarType(sec)
    {  }

    /**
     * Timeを表すスカラー変数をコピーして生成する.
     * @param rhs コピー元の変数
     */
    TimeBase(const typename TimeBaseTraits<T>::ScalarType& rhs)
        : TimeBaseTraits<T>::ScalarType(rhs)
    {  }

    /**
      * sec単位の値を得る.
      * @return sec
      */
    const T& ToSec() const
    {
        return this->X();
    }

    /**
      * secからTimeを表すスカラー変数を生成する.
      * @param sec sec
      * @return Timeを表すスカラー変数
      */
    static TimeBase FromSec(const T& sec)
    {
        return TimeBase(sec);
    }
};

/**
 * よく使うTimeの型.
 */
typedef TimeBase<NormalValueType> Time;

}

#endif /* DIMPRO_TIME_HPP_ */
